# -*- coding: utf-8 -*-
import logging

from pyramid.config import Configurator

log = logging.getLogger(__name__)


def includeme(config):
    log.debug('Including pyramid_restlogging')

    config.add_route(
        'clientlogs',
        '/clientlogs/{logger_name:[\d\w\.:]+}/{log_level:debug|info|warning|error|critical}',
        request_method='POST',
        )
    config.scan('pyramid_restlogging.views')
    config.commit()


def main(global_config, **settings):
    # global_config is not used here,  see Paste doc
    config = Configurator(settings=settings)
    config.include(includeme)
    return config.make_wsgi_app()



# -*- coding: utf-8 -*-
import logging

from pyramid.view import view_config
from pyramid.httpexceptions import HTTPOk

log = logging.getLogger(__name__)


@view_config(route_name='clientlogs')
def log_view(request):
    try:
        # TODO use the 'ignore' or 'replace' error handler
        # XXX check if Pyramid does not already decode
        message = unicode(request.body, request.charset)
    except UnicodeDecodeError:
        message = request.body
    logger_name = request.matchdict['logger_name']
    log_level = request.matchdict['log_level']
    logger = logging.getLogger(logger_name)
    # checking for valid log level is already done by the route
    getattr(logger, log_level)(message)
    return HTTPOk()



# -*- coding: utf-8 -*-
import unittest

import mock
from pyramid.testing import DummyRequest


class TestIncludeme(unittest.TestCase):

    def test_basic(self):
        from pyramid_restlogging import includeme
        config = mock.Mock()

        includeme(config)

        config.add_route.assert_called_with('clientlogs', mock.ANY,
                                            request_method='POST')
        config.scan.assert_called_with('pyramid_restlogging.views')
        config.commit.assert_called_with()


class TestLogView(unittest.TestCase):

    def test_nominal_case(self):
        from pyramid_restlogging.views import log_view
        req = DummyRequest()
        req.matchdict = {
            'logger_name': 'com.ludia:Application',
            'log_level': 'debug',
            }
        req.body = 'beware of the dog.'

        log_view(req)



# encoding: utf-8
import unittest

import webtest


class TestLogView(unittest.TestCase):

    maxDiff = None

    def setUp(self):
        from pyramid_restlogging import main
        self.app = webtest.TestApp(main(None))

    def test_basic(self):
        body = 'Error: no more spam'
        self.app.post('/clientlogs/spam/error', body, status=200)

    def test_wrong_encoding(self):
        body = u'Unknown user: Éric'.encode('latin-1')
        headers = {'Content-Type': 'text/plain; charset=utf-8'}
        self.app.post('/clientlogs/spam/error', body, headers, status=200)

    def test_invalid_loglevel(self):
        body = 'Error: too much ham'
        self.app.post('/clientlogs/spam/impossible', body, status=404)



from setuptools import setup

setup_requires = [
    'd2to1',
    'nose',
    'nosexcover',
    'coverage',
    'mock',
    'webtest',
    'sphinx',
]

setup(
    setup_requires=setup_requires,
    d2to1=True,
    entry_points="""\
        [paste.app_factory]
        main = pyramid_restlogging:main
        """
)
